@@ -3,14 +3,15 @@
 """
 Configuration functions for the proteus package for Tryton.
 """
-from __future__ import with_statement
-__all__ = ['set_trytond', 'set_xmlrpc', 'get_config']
-import xmlrpclib
-import threading
-from decimal import Decimal
+
 import datetime
 import os
-import urlparse
+import threading
+import urllib.parse
+import xmlrpc.client
+from decimal import Decimal
+
+__all__ = ['set_trytond', 'set_xmlrpc', 'get_config']
 
 
 def dump_decimal(self, value, write):
@@ -22,7 +23,7 @@ def dump_decimal(self, value, write):
 
 def dump_bytes(self, value, write):
     self.write = write
-    value = xmlrpclib.Binary(value)
+    value = xmlrpc.client.Binary(value)
     value.encode(self)
     del self.write
 
@@ -52,13 +53,13 @@ def dump_timedelta(self, value, write):
         }
     self.dump_struct(value, write)
 
-xmlrpclib.Marshaller.dispatch[Decimal] = dump_decimal
-xmlrpclib.Marshaller.dispatch[datetime.date] = dump_date
-xmlrpclib.Marshaller.dispatch[datetime.time] = dump_time
-xmlrpclib.Marshaller.dispatch[datetime.timedelta] = dump_timedelta
+xmlrpc.client.Marshaller.dispatch[Decimal] = dump_decimal
+xmlrpc.client.Marshaller.dispatch[datetime.date] = dump_date
+xmlrpc.client.Marshaller.dispatch[datetime.time] = dump_time
+xmlrpc.client.Marshaller.dispatch[datetime.timedelta] = dump_timedelta
 if bytes != str:
-    xmlrpclib.Marshaller.dispatch[bytes] = dump_bytes
-xmlrpclib.Marshaller.dispatch[bytearray] = dump_bytes
+    xmlrpc.client.Marshaller.dispatch[bytes] = dump_bytes
+xmlrpc.client.Marshaller.dispatch[bytearray] = dump_bytes
 
 
 class XMLRPCDecoder(object):
@@ -96,7 +97,7 @@ def end_struct(self, data):
     self._stack[mark:] = [dct]
     self._value = 0
 
-xmlrpclib.Unmarshaller.dispatch['struct'] = end_struct
+xmlrpc.client.Unmarshaller.dispatch['struct'] = end_struct
 
 _CONFIG = threading.local()
 _CONFIG.current = None
@@ -137,6 +138,11 @@ class Config(object):
         self._context.update(kwargs)
         return ctx_manager
 
+    def reset_context(self):
+        ctx_manager = ContextManager(self)
+        self._context = {}
+        return ctx_manager
+
     def get_proxy(self, name):
         raise NotImplementedError
 
@@ -156,6 +162,7 @@ class _TrytondMethod(object):
         from trytond.rpc import RPC
         from trytond.tools import is_instance_method
         from trytond.transaction import Transaction
+        from trytond.worker import run_task
 
         if self._name in self._object.__rpc__:
             rpc = self._object.__rpc__[self._name]
@@ -180,8 +187,10 @@ class _TrytondMethod(object):
                 else:
                     result = [rpc.result(meth(i, *args, **kwargs))
                         for i in inst]
-            if not rpc.readonly:
-                transaction.commit()
+            transaction.commit()
+        while transaction.tasks:
+            task_id = transaction.tasks.pop()
+            run_task(self._config.database_name, task_id)
         return result
 
 
@@ -217,7 +226,7 @@ class TrytondConfig(Config):
         self.database = database
         database_name = None
         if database:
-            uri = urlparse.urlparse(database)
+            uri = urllib.parse.urlparse(database)
             database_name = uri.path.strip('/')
         if not database_name:
             database_name = os.environ['DB_NAME']
@@ -232,9 +241,10 @@ class TrytondConfig(Config):
         with Transaction().start(self.database_name, 0) as transaction:
             User = self.pool.get('res.user')
             transaction.context = self.context
-            self.user = User.search([
-                ('login', '=', user),
-                ], limit=1)[0].id
+            with transaction.set_context(active_test=False):
+                self.user = User.search([
+                    ('login', '=', user),
+                    ], limit=1)[0].id
             with transaction.set_user(self.user):
                 self._context = User.get_preferences(context_only=True)
     __init__.__doc__ = object.__init__.__doc__
@@ -297,7 +307,7 @@ class XmlrpcConfig(Config):
     def __init__(self, url, **kwargs):
         super(XmlrpcConfig, self).__init__()
         self.url = url
-        self.server = xmlrpclib.ServerProxy(
+        self.server = xmlrpc.client.ServerProxy(
             url, allow_none=1, use_datetime=1, **kwargs)
         # TODO add user
         self.user = None
